@@ -1,6 +1,8 @@
 ## multi user blaze server code.  This module contains extra routes for the blaze
 ## data backend
 
+from __future__ import absolute_import
+
 import warnings
 import logging
 
@@ -9,7 +11,7 @@ logger = logging.getLogger(__name__)
 def get_blueprint(config_file=None):
     retval = None
     try:
-        from . import views
+        from . import views; views
         import mbs.app
         retval = mbs.app.mbsbp
         mbs.app.setup_app(config_file=config_file)
