@@ -5,8 +5,8 @@ from numpy import pi, arange, sin
 from bokeh.browserlib import view
 from bokeh.document import Document
 from bokeh.embed import file_html
-from bokeh.glyphs import Circle
-from bokeh.objects import (
+from bokeh.models.glyphs import Circle
+from bokeh.models import (
     Plot, DataRange1d, LinearAxis, ColumnDataSource, PanTool, WheelZoomTool
 )
 from bokeh.resources import INLINE
@@ -18,8 +18,8 @@ source = ColumnDataSource(
     data=dict(x=x, y=y)
 )
 
-xdr = DataRange1d(sources=[source.columns("x")])
-ydr = DataRange1d(sources=[source.columns("y")])
+xdr = DataRange1d()
+ydr = DataRange1d()
 
 plot = Plot(x_range=xdr, y_range=ydr, min_border=80)
 
